@@ -1,6 +1,6 @@
 <?php
 /* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
-/* @(#) $Header: /sources/phpprintipp/phpprintipp/ExtendedPrintIPP.php,v 1.10 2006/01/06 00:08:53 harding Exp $
+/* @(#) $Header: /sources/phpprintipp/phpprintipp/ExtendedPrintIPP.php,v 1.12 2006/01/16 22:16:17 harding Exp $
  *
  * Version: 0.3
  * Class PrintIPP - Send extended IPP requests.
@@ -34,6 +34,7 @@
     References needed to debug / add functionnalities:
         - RFC 2910
         - RFC 2911
+        - RFC 3380
         - RFC 3995
         - RFC 3996
         - RFC 3997
@@ -80,7 +81,7 @@ class ExtendedPrintIPP extends PrintIPP {
                               "Data" => $this->output);
             
             
-        if (self::_sendHttp ($post_values,'/printers/')) {
+        if (self::_sendHttp ($post_values,$this->paths['printers'])) {
             
             if(self::_parseServerOutput()) {
                 $this->_getJobId();
@@ -170,7 +171,7 @@ class ExtendedPrintIPP extends PrintIPP {
         $post_values = array( "Content-Type"=>"application/ipp",
                               "Data"=>$this->output);
             
-        if (self::_sendHttp ($post_values,'/admin/'))
+        if (self::_sendHttp ($post_values,$this->paths['admin']))
             self::_parseServerOutput();
         
 
@@ -265,28 +266,9 @@ class ExtendedPrintIPP extends PrintIPP {
                              . $timeout;
                              
         $jobattributes = '';
-        foreach ($this->job_tags as $key => $value) {
-            if (array_key_exists('value',$value))
-                $jobattributes .= $value['systag']
-                                . self::_giveMeStringLength($key)
-                                . $key
-                                . self::_giveMeStringLength($value['value'])
-                                . $value['value'];
-        }
-        reset ($this->job_tags);
+        $operationattributes = '';
+        self::_buildValues($operationattributes,$jobattributes);
         
-        $printerattributes = '';
-        foreach ($this->printer_tags as $key => $value) {
-            if (array_key_exists('value',$value))
-                $printerattributes .= $value['systag']
-                                . self::_giveMeStringLength($key)
-                                . $key
-                                . self::_giveMeStringLength($value['value'])
-                                . $value['value'];
-        }
-        reset ($this->printer_tags);
- 
- 
         $this->stringjob = chr(0x01) . chr(0x01) // 1.1  | version-number
                          . chr(0x00) . chr (0x05) // Create-Job | operation-id
                          . $this->meta->operation_id //           request-id
@@ -298,7 +280,7 @@ class ExtendedPrintIPP extends PrintIPP {
                          . $this->meta->jobname
                          . $this->meta->fidelity
                          . $this->meta->timeout
-                         . $printerattributes
+                         . $operationattributes
                          . chr(0x02) // start job-attributes | job-attributes-tag
                          . $this->meta->copies
                          . $this->meta->sides
@@ -317,7 +299,7 @@ class ExtendedPrintIPP extends PrintIPP {
         $post_values = array( "Content-Type"=>"application/ipp",
                               "Data"=>$this->output);
             
-        if (self::_sendHttp ($post_values,'/printers/'))
+        if (self::_sendHttp ($post_values,$this->paths['printers']))
             if(self::_parseServerOutput()) {
                 $this->_getJobId();
                 $this->_getJobUri();
@@ -398,7 +380,7 @@ class ExtendedPrintIPP extends PrintIPP {
             
             }
             
-        if (self::_sendHttp ($post_values,'/printers/')) {
+        if (self::_sendHttp ($post_values,$this->paths['printers'])) {
         
             if(self::_parseServerOutput()) {
                 $this->_getJobId();
@@ -451,7 +433,7 @@ class ExtendedPrintIPP extends PrintIPP {
             $post_values = array( "Content-Type" => "application/ipp",
                                   "Data" => $this->output);
             
-        if (self::_sendHttp ($post_values,'/printers/')) {
+        if (self::_sendHttp ($post_values,$this->paths['printers'])) {
         
             if(self::_parseServerOutput()) {
                 $this->_getJobId();
@@ -548,7 +530,7 @@ class ExtendedPrintIPP extends PrintIPP {
         $post_values = array( "Content-Type"=>"application/ipp",
                               "Data"=>$this->output);
             
-        if (self::_sendHttp ($post_values,'/admin/'))
+        if (self::_sendHttp ($post_values,$this->paths['admin']))
             self::_parseServerOutput();
         
 
@@ -631,7 +613,7 @@ class ExtendedPrintIPP extends PrintIPP {
         $post_values = array( "Content-Type"=>"application/ipp",
                               "Data"=>$this->output);
             
-        if (self::_sendHttp ($post_values,'/admin/'))
+        if (self::_sendHttp ($post_values,$this->paths['admin']))
             self::_parseServerOutput();
         
 
@@ -717,7 +699,7 @@ class ExtendedPrintIPP extends PrintIPP {
         $post_values = array( "Content-Type"=>"application/ipp",
                               "Data"=>$this->output);
             
-        if (self::_sendHttp ($post_values,'/jobs/')) 
+        if (self::_sendHttp ($post_values,$this->paths['jobs'])) 
             self::_parseServerOutput();
         
         if (isset($this->serveroutput) && isset($this->serveroutput->status)) {
@@ -788,7 +770,7 @@ class ExtendedPrintIPP extends PrintIPP {
         $post_values = array( "Content-Type"=>"application/ipp",
                               "Data"=>$this->output);
             
-        if (self::_sendHttp ($post_values,'/jobs/')) 
+        if (self::_sendHttp ($post_values,$this->paths['jobs'])) 
             self::_parseServerOutput();
         
         if (isset($this->serveroutput) && isset($this->serveroutput->status)) {
@@ -841,16 +823,7 @@ class ExtendedPrintIPP extends PrintIPP {
         
        
         $jobattributes = '';
-        foreach ($this->job_tags as $key => $value) {
-            if (array_key_exists('value',$value))
-                $jobattributes .= $value['systag']
-                                . self::_giveMeStringLength($key)
-                                . $key
-                                . self::_giveMeStringLength($value['value'])
-                                . $value['value'];
-        }
-        reset ($this->job_tags);
-        
+        self::_buildValues ($operationattributes,$jobattributes); 
  
         
         $this->stringjob = chr(0x01) . chr(0x01) // 1.1  | version-number
@@ -874,7 +847,7 @@ class ExtendedPrintIPP extends PrintIPP {
         $post_values = array( "Content-Type"=>"application/ipp",
                               "Data"=>$this->output);
             
-        if (self::_sendHttp ($post_values,'/jobs/')) 
+        if (self::_sendHttp ($post_values,$this->paths['jobs'])) 
             self::_parseServerOutput();
         
         if (isset($this->serveroutput) && isset($this->serveroutput->status)) {
@@ -901,6 +874,106 @@ class ExtendedPrintIPP extends PrintIPP {
     }
     // }}}
     
+    // {{{ setJobAttributes ($job_uri,$deleted_attributes=array())
+    public function setJobAttributes ($job_uri,$deleted_attributes=array()) {
+    
+        $this->jobs = array_merge($this->jobs,array(""));
+        $this->jobs_uri = array_merge($this->jobs_uri,array(trim($job_uri)));
+
+        self::_setOperationId();
+        $this->parsed = array();
+        unset($this->printer_attributes);
+           
+        if (!isset($this->setup->charset))
+            self::setCharset('us-ascii');
+            
+        if (!isset($this->setup->language))
+            self::setLanguage('en_us');
+
+        if (!isset($this->meta->username))
+            self::setUserName();
+        
+        if (!isset($this->meta->message))
+            $this->meta->message = '';
+
+            
+        if (!isset($this->meta->copies))
+            $this->meta->copies = '';
+
+        if (!isset($this->meta->sides))
+            $this->meta->sides = '';
+        
+        if (!isset($this->meta->page_ranges))
+            $this->meta->page_ranges = '';
+  
+        self::_setJobUri($job_uri);
+        
+       
+        $jobattributes = '';
+        self::_buildValues ($operationattributes,$jobattributes); 
+        
+        $this->meta->deleted_attributes = "";
+        for ($i = 0 ; $i < count($deleted_attributes) ; $i++)
+                $this->meta->deleted_attributes .= chr(0x16) // out-of-band value
+                                                . self::_giveMeStringLength($deleted_attributes[$i])
+                                                . $deleted_attributes[$i]
+                                                . chr(0x0).chr(0x0); // value-length = 0;
+                
+        
+        $this->stringjob = chr(0x01) . chr(0x01) // 1.1  | version-number
+                         . chr(0x00) . chr (0x14) // Set-Job-Attributes | operation-id
+                         . $this->meta->operation_id //           request-id
+                         . chr(0x01) // start operation-attributes | operation-attributes-tag
+                         . $this->meta->charset
+                         . $this->meta->language
+                         . $this->meta->job_uri
+                         . $this->meta->username
+                         . $this->meta->message
+                         . chr(0x02) // start job-attributes
+                         . $jobattributes // setteds by setAttribute($attribute,$value)
+                         . $this->meta->copies
+                         . $this->meta->sides
+                         . $this->meta->page_ranges
+                         . $this->meta->deleted_attributes
+                         . chr(0x03); // end-of-attributes | end-of-attributes-tag
+                         
+        self::_putDebug(sprintf(_("String sent to the server is:\n%s\n"), $this->stringjob));
+        
+        self::_putDebug(sprintf(_("set job attributes for job %s\n"),$job_uri)); 
+            
+        $this->output = $this->stringjob;
+          
+        $post_values = array( "Content-Type"=>"application/ipp",
+                              "Data"=>$this->output);
+            
+        if (self::_sendHttp ($post_values,$this->paths['jobs'])) 
+            self::_parseServerOutput();
+        
+        if (isset($this->serveroutput) && isset($this->serveroutput->status)) {
+            
+            $this->status = array_merge($this->status,array($this->serveroutput->status));
+            
+            if ($this->serveroutput->status == "successfull-ok")
+                self::_errorLog(sprintf(_("set job attributes for job %s: "),$job_uri)
+                            .$this->serveroutput->status,3);
+            else
+                 self::_errorLog(sprintf(_("set job attributes for job %s: "),$job_uri)
+                                             .$this->serveroutput->status,1);
+            $this->last_job = $job_uri;
+            $this->jobs_uri[count($this->jobs_uri) - 1] = $job_uri;
+            return $this->serveroutput->status; 
+            }    
+            
+        $this->status = array_merge($this->status,array("OPERATION FAILED"));
+        self::_errorLog(date("Y-m-d H:i:s : ")
+                        .basename($_SERVER['PHP_SELF'])
+                        .sprintf(_("set job attributes for job %s: OPERATION FAILED"),
+                                     $job_uri),3);
+            
+    return false;
+    }
+    // }}}
+    
     
     // }}}
 
@@ -994,26 +1067,8 @@ class ExtendedPrintIPP extends PrintIPP {
             $this->meta->page_ranges = '';
          
         $jobattributes = '';
-        foreach ($this->job_tags as $key => $value) {
-            if (array_key_exists('value',$value))
-                $jobattributes .= $value['systag']
-                                . self::_giveMeStringLength($key)
-                                . $key
-                                . self::_giveMeStringLength($value['value'])
-                                . $value['value'];
-        }
-        reset ($this->job_tags);
-        
-        $printerattributes = '';
-        foreach ($this->printer_tags as $key => $value) {
-            if (array_key_exists('value',$value))
-                $printerattributes .= $value['systag']
-                                . self::_giveMeStringLength($key)
-                                . $key
-                                . self::_giveMeStringLength($value['value'])
-                                . $value['value'];
-        }
-        reset ($this->printer_tags);
+        $operationattributes = '';
+        self::_buildValues($operationattributes,$jobattributes);
          
         self::_setOperationId();
        
@@ -1033,7 +1088,7 @@ class ExtendedPrintIPP extends PrintIPP {
                          . $this->meta->fidelity
                          . $this->meta->document_name
                          . $this->meta->document_uri
-                         . $printerattributes
+                         . $operationattributes
                          . chr(0x02) // start job-attributes | job-attributes-tag
                          . $this->meta->copies
                          . $this->meta->sides
@@ -1099,15 +1154,7 @@ class ExtendedPrintIPP extends PrintIPP {
             $this->meta->page_ranges = '';
        
         $operationattributes = '';
-        foreach ($this->printer_tags as $key => $value) {
-            if (array_key_exists('value',$value))
-                $operationattributes .= $value['systag']
-                                . self::_giveMeStringLength($key)
-                                . $key
-                                . self::_giveMeStringLength($value['value'])
-                                . $value['value'];
-        }
-        reset ($this->printer_tags);
+        self::_buildValues($operationattributes,$jobattributes);
         
         self::_setOperationId();
 
@@ -1205,15 +1252,7 @@ class ExtendedPrintIPP extends PrintIPP {
             $this->meta->page_ranges = '';
        
         $operationattributes = '';
-        foreach ($this->printer_tags as $key => $value) {
-            if (array_key_exists('value',$value))
-                $operationattributes .= $value['systag']
-                                . self::_giveMeStringLength($key)
-                                . $key
-                                . self::_giveMeStringLength($value['value'])
-                                . $value['value'];
-        }
-        reset ($this->printer_tags);
+        self::_buildValues($operationattributes,$jobattributes);
         
         self::_setOperationId();
 
