@@ -47,6 +47,7 @@ $language = "fr_fr";
 $user = "test"; // valid user from lpadmin group
 $password = "test"; // his password
 $document_uri = "http://localhost/";
+$debug_level = 1; // 1: silent; 3: very verbose
 
 error_reporting(E_ALL);
 /******************
@@ -83,7 +84,7 @@ function __autoload($class_name) {
 //$ipp = new ExtendedPrintIPP(); // extended IPP
 $ipp = new CupsPrintIPP(); // extended IPP with CUPS specific stuff
 
-//$ipp->debug_level = 3; // Debugging very verbose
+$ipp->debug_level = $debug_level; // Debugging
 
 $ipp->setLog('/tmp/printipp','file',1); // logging almost quiet
 $ipp->setLog('/tmp/printipp','file',2); // logging verbose
@@ -109,15 +110,21 @@ $ipp->setAuthentification($user,$password); // username & password
 $j = 0;
 $test = 1;
 
-
 /* printing a large file */
 
 echo "<br /><br /><br />TEST ".$test++."<br />";
 echo "OPERATION  ".$j++."<br />";
+$ipp->setAttribute("printer-resolution","1440x720dpi");
+#$ipp->setAttribute("job-sheets", array("confidential","secret"));
+$ipp->setAttribute("job-billing", "Thomas");
+$ipp->setAttribute("print-quality", "high");
+//$ipp->setCopies(2);
 $ipp->setData('testfiles/photo.jpg');
+$ipp->setJobName('photo.jpg',true);
 echo "Printing a large file: ". $ipp->PrintJob() . "<br />";
 printf('$ipp->status[0] = %s <br /><br />', $ipp->status[0]);
-
+$ipp->unsetAttribute("job-billing");
+$ipp->setJobName();
 
 /* getting printer's attributes */
 
@@ -125,51 +132,10 @@ echo "<br /><br /><br />TEST ".$test++."<br />";
 echo "OPERATION  ".$j++."<br />";
 echo "Getting Printer's attributes: ". $ipp->getPrinterAttributes() . "<br />";
 printf('$ipp->status[%s] = %s <br /><br />', count($ipp->status) -1,$ipp->status[count($ipp->status) -1]);
+$timestamp = $ipp->printer_attributes->printer_up_time->_value0;
+echo "Printer up-time: ".date('Y-m-d H:i:s',$timestamp)."<br />";
 echo "Printer's attributes :<pre>\n"; print_r($ipp->printer_attributes); echo "</pre>";
 
-/* getting printers (CUPS extention) */
-
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
-
-echo "OPERATION ".$j++."<br />";
-echo "Available printers [CUPS EXTENTION]:<pre>" ; 
-echo $ipp->getPrinters() . "\n";
-print_r($ipp->available_printers);
-echo "</pre>";
-
-
-for ($i = 0 ; $i < count($ipp->available_printers) ; $i ++) {
-$ipp->setPrinterURI($ipp->available_printers[$i]);
-echo "OPERATION ".$j ++."<br />";
-$ipp->getPrinterAttributes();
-echo "Printer attributes for printer $i:<pre>\n"; print_r($ipp->printer_attributes); echo "</pre>";
-if (isset($ipp->printer_attributes->printer_type->_value2)
-        && ($ipp->printer_attributes->printer_type->_value2) == 'print-black')
-    echo "The printer can print black<br />\n";
-if (isset($ipp->printer_attributes->printer_type->_value3)
-        && ($ipp->printer_attributes->printer_type->_value3) == 'print-color')
-    echo "The printer can print color<br />\n";
-
-    echo "Printer State: ".$ipp->printer_attributes->printer_state->_value0."<br />";
-    echo "Printer State message: ".$ipp->printer_attributes->printer_state_message->_value0."<br />";
-    echo "Document formats supported:<br /><pre>";
-
-    $pointer = "_value0";
-    for ($k = 0 ; isset($ipp->printer_attributes->document_format_supported->$pointer); $k++) {
-        echo $ipp->printer_attributes->document_format_supported->$pointer . "\n";
-        $pointer = "_value" . ($k + 1);
-        }
-    echo "</pre>";
-
-echo "------- END FOR PRINTER $i -------------<br /n>";
-}
-
-
-/* resetting printer uri to those configured */
-
-$ipp->setPrinterURI($printer_uri);
-
-
 /* printing a string */
 
 echo "<br /><br /><br /> TEST ".$test ++."<br />";
@@ -180,7 +146,7 @@ $ipp->setData("This is a text string");
 echo "Print String: ".$ipp->printJob()."</br />";
 echo "Job Attributes:<pre>\n" ; print_r($ipp->job_attributes) ; echo "\n</pre>\n";
 
-// for restart job operation
+// for restart job operation (later)
 $first_job = $ipp->last_job;
 
 /* printing a text file */
@@ -264,16 +230,41 @@ echo "<pre>";print_r($ipp->jobs_attributes); echo "</pre>";
 echo "<br /><br /><br /> TEST ".$test ++."<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
-echo "Getting last job's attributes: ".$ipp->getJobAttributes($ipp->last_job,false,'job-template')."<br />";
+echo "Getting last job's attributes: ".$ipp->getJobAttributes($ipp->last_job,false,'all')."<br />";
                     
 $job_state = $ipp->job_attributes->job_state->_value0;
 echo "Job-State: $job_state<br />";
 
-$job_state_reasons = $ipp->job_attributes->job_state_reasons->_value0;
+$pointer = "_value0";
+$job_state_reasons = '';
+for ($k = 0 ; isset($ipp->job_attributes->job_state_reasons->$pointer) ; $k++) {
+    $job_state_reasons .= $ipp->job_attributes->job_state_reasons->$pointer .", ";
+    $pointer = "_value".($k + 1);
+    }
 echo "Job-State-Reasons: $job_state_reasons<br />";
  
 echo "<pre>";print_r($ipp->job_attributes); echo "</pre>";
 
+/* setting job's attributes to new values */
+
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
+echo "deleting print-quality attribute<br />";
+$unset_attributes = array("print-quality");//,"printer-resolution");
+echo "setting copies to 2<br />";
+$ipp->setCopies(2);
+echo "setting resolution to 320x200dpi<br />";
+$ipp->setAttribute('printer-resolution','320x200dpi');
+echo "Setting last job's attributes: ".$ipp->setJobAttributes($ipp->last_job,$unset_attributes)."<br />";
+echo "OPERATION ". $j ++ ."<br />";
+echo "Getting job's attributes: ".$ipp->getJobAttributes($ipp->last_job)."<br />";
+$resolution = $ipp->job_attributes->printer_resolution->_value0;
+echo "Resolution: $resolution<br />";
+$print_quality = $ipp->job_attributes->print_quality->_value0;
+echo "Print-Quality: $print_quality<br />";
+ 
+echo "<pre>";print_r($ipp->job_attributes); echo "</pre>";
 
 
 /* printing selected pages from a document */
@@ -405,14 +396,14 @@ echo "<br /><br /><br /> TEST ".$test ++."<br />";
 $ipp->setMimeMediaType("application/x-foobar");
 
 echo "OPERATION ". $j ++ ."<br />";
-echo "Validate-Job: ".$ipp->validateJob()."<br />";
+echo "Validate-Job for a document with file format 'application/x-foobar': ".$ipp->validateJob()."<br />";
 
-foreach ($ipp->job_attributes as $name => $job_attribute) {
-    if ($job_attribute->_range == 'unsupported-attributes')
-        printf('%s "%s": unsupported attribute<br />',$name,$job_attribute->_value0);
+foreach ($ipp->attributes as $name => $attribute) {
+    if ($attribute->_range == 'unsupported-attributes')
+        printf('%s "%s": unsupported attribute<br />',$name,$attribute->_value0);
 }
-reset($ipp->job_attributes);
-echo "Details:<pre>"; print_r($ipp->job_attributes) ; echo "</pre>";
+reset($ipp->attributes);
+echo "Details:<pre>"; print_r($ipp->attributes) ; echo "</pre>";
 
 
 /* Printing then holding a job */
@@ -495,8 +486,89 @@ echo "Getting printer's attributes: ".$ipp->getPrinterAttributes()."<br />";
 echo "Printer State: ".$ipp->printer_attributes->printer_state->_value0."<br />";
 
 
-/* send a buggy request */
+/* getting only default printer's uri (CUPS extention) */
 
+echo "<br /><br /><br />TEST ".$test++."<br />";
+echo "OPERATION ".$j ++."<br />";
+
+printf("Default printer URI [CUPS specific]: %s<pre>\n",$ipp->cupsGetDefaults(array('printer-uri-supported')));
+printf("Default printer URI is: %s<br />",$ipp->printer_attributes->printer_uri_supported->_value0);
+print_r($ipp->printer_attributes); echo "</pre>";
+
+
+
+/* getting default printer attributes (CUPS extension) */
+
+echo "<br /><br /><br />TEST ".$test++."<br />";
+echo "OPERATION ".$j ++."<br />";
+echo "Getting default printer attributes [CUPS specific]<br /><pre>\n";
+$ipp->cupsGetDefaults(array('all'));
+
+print_r($ipp->printer_attributes);
+
+echo "</pre>\nPrinter historic for default printer:\n"; 
+    
+$histo = $ipp->printer_attributes->printer_state_history->_value1;
+    
+    $idx_histo = "_indice0";
+    for ($idx = 0 ; isset($histo->$idx_histo) ; $idx ++) {
+    echo "<h3>next event:</h3>\n";
+            foreach ($histo->$idx_histo as $key => $value) {
+                if (is_object($value))
+                    if ($key != 'printer_state_time') 
+                        printf("%s: %s<br />",$key,$value->_value0);
+                    else
+                        printf("%s: %s<br />",$key,date('Y-m-d H:i:s',$value->_value0));
+                    $idx_key = "_key". ($key + 1);
+                }
+    $idx_histo = '_indice'. ($idx + 1);
+    }
+echo "<br /><br /><br />\n";
+
+
+/* getting printers (vendor extention) (currently CUPS only)*/
+
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ".$j++."<br />";
+echo "Available printers [CUPS EXTENTION]:<pre>" ; 
+echo $ipp->getPrinters() . "\n"; // there is an alias "cupsGetPrinters" for CUPS.
+print_r($ipp->available_printers);
+echo "</pre>";
+echo "Printers's attributes<pre>\n";
+print_r ($ipp->printers_attributes);
+echo "</pre>\n";
+
+
+for ($i = 0 ; $i < count($ipp->available_printers) ; $i ++) {
+$ipp->setPrinterURI($ipp->available_printers[$i]);
+echo "OPERATION ".$j ++."<br />";
+$ipp->getPrinterAttributes();
+echo "Printer attributes for printer $i:<pre>\n"; print_r($ipp->printer_attributes); echo "</pre>";
+if (isset($ipp->printer_attributes->printer_type->_value2)
+        && ($ipp->printer_attributes->printer_type->_value2) == 'print-black')
+    echo "The printer can print black<br />\n";
+if (isset($ipp->printer_attributes->printer_type->_value3)
+        && ($ipp->printer_attributes->printer_type->_value3) == 'print-color')
+    echo "The printer can print color<br />\n";
+
+    echo "Printer State: ".$ipp->printer_attributes->printer_state->_value0."<br />";
+    echo "Printer State message: ".$ipp->printer_attributes->printer_state_message->_value0."<br />";
+    echo "Document formats supported:<br /><pre>";
+
+    $pointer = "_value0";
+    for ($k = 0 ; isset($ipp->printer_attributes->document_format_supported->$pointer); $k++) {
+        echo $ipp->printer_attributes->document_format_supported->$pointer . "\n";
+        $pointer = "_value" . ($k + 1);
+        }
+    echo "</pre>";
+
+echo "------- END FOR PRINTER $i -------------<br /n>";
+}
+
+
+/* send a buggy request */
+/*
 echo "<br /><br /><br /> TEST ".$test ++."<br />";
 echo "OPERATION ". $j ++ ."<br />";
 $ipp->setData("This is an error : nothing printed\n");
@@ -506,8 +578,9 @@ echo "Sending a buggy request. status: ".$ipp->printJob()."<br />";
 $ipp->resetError("request_body_malformed");
 
 
-/**/
 
+/**/
+/*
 echo "END OF OPERATIONS <br /><br /><br />";
 
 /* get informations about jobs and status */
